<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2008 Kurt Kunig <kurt.kunig@gmx.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

// wg. TYPO3 V6.x - require_once(PATH_tslib.'class.tslib_pibase.php');


/**
 * Plugin 'Table from CSV' for the 'kk_csv2table' extension.
 *
 * @author	Kurt Kunig <kurt.kunig@gmx.de>
 * @package	TYPO3
 * @subpackage	tx_kkcsv2table
 */
class tx_kkcsv2table_pi1 extends \TYPO3\CMS\Frontend\Plugin\AbstractPlugin {
	var $prefixId      = 'tx_kkcsv2table_pi1';		// Same as class name
	var $scriptRelPath = 'pi1/class.tx_kkcsv2table_pi1.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'kk_csv2table';	// The extension key.
	var $pi_checkCHash = true;
	var $filebasepath	 = "uploads/tx_kkcsv2table/";

	// multilanguage
	var $langArr;
	var $sys_language_mode;


   var $debug = false;
   var $utf8 = true;
   var $col_delimiter = ';';
   var $text_delimiter = '"';

   var $oddEvenArray = array(
      '0' => 'tr-headline',
      '1' => 'tr-odd',
      '2' => 'tr-even',
      );



	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function main($content,$conf)	{
		global $TSFE,$LANG;
		$this->conf=$conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		
		$this->debug = $this->conf['debug'];
      if ($this->debug) print "<style>body {background:white !important;}</style><h4>START MAIN-function</h4>";
		// test if language version available
		// Die im Backend definierten Sprachen laden und
		// die Handhabung fr bersetzungen ermitteln.
		$lres = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_language', '1=1' . $this->cObj->enableFields('sys_language'));
		$this->langArr = array();
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($lres)) {
		    $this->langArr[$row['uid']] = $row;
		}

      // Check if php-function 'str_getcsv' exists and create an own one:
      if(!function_exists('str_getcsv')) {
          if ($this->debug) print "<h1>php-function 'str_getcsv' does not exist!?</h1>";
          function str_getcsv($input, $delimiter = ",", $enclosure = '"', $escape = "\\") {
              $fp = fopen("php://memory", 'r+');
              fputs($fp, $input);
              rewind($fp);
              $data = fgetcsv($fp, null, $delimiter, $enclosure); // $escape only got added in 5.3.0
              fclose($fp);
              return $data;
          }
      }


		// get sys_language_mode
		$this->sys_language_mode = $this->conf['sys_language_mode']?$this->conf['sys_language_mode']:$GLOBALS['TSFE']->sys_language_mode;

		$langUID = $TSFE->config['config']['sys_language_uid'];
		// if config.sys_language_uid is not set, $langUID = 0
		if(empty($langUID)) $langUID = '0';

		// flexform Integration
		$this->pi_initPIflexform('tx_kkcsv2table_flex'); // Init and get the flexform data of the plugin

      // Read the flexform parameters into internal variables:
  		$csv_file = $this->filebasepath . $this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'csv_file', 'sDEF');
  		// A dedicated file path has a higher priority:
      $k = $this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'csvDedicatedFilePath', 'sDEF');
      $k = str_replace('../', '', $k);
      if (!empty($k) && $k != '') {
         if (substr($k,-4) == '.csv' || substr($k,-4) == '.txt') {
            $csv_file = 'fileadmin/' . $k;
            if (substr($k,0,1) == '/') $csv_file = 'fileadmin' . $k;
         } else {
            return '<p style="color:red; font-weight:bold;" class="'.$extKey.'">Only csv- or txt-files are allowed!</p>';
         }
      }

      $this->col_delimiter = $this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'col_delimiter', 'sDEF');
      if ($this->col_delimiter == '' || empty($this->col_delimiter)) $this->col_delimiter = ";";
      if (strtoupper($this->col_delimiter) == "[T]") $this->col_delimiter = chr(9);
      if ($this->debug) print '<p style="color:#2222ad;">$this->col_delimiter(hex) = '.chunk_split(bin2hex($this->col_delimiter), 2, ' ') .' ('.$this->col_delimiter.')</p>';

      $this->text_delimiter = $this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'text_delimiter', 'sDEF');
      if ($this->text_delimiter == '' || empty($this->text_delimiter)) $this->text_delimiter = '"';

 		$table_summary = trim($this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'table_summary', 'sDEF'));
 		$table_class = trim($this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'table_class', 'sDEF'));
  		$header_data = $this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'header_data', 'sDEF');
  		$own_header = trim($this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'own_header', 'sDEF'));
  		$this->csv_code = trim($this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'csv_code', 'sDEF'));
  		$this->utf8_encoded = trim($this->pi_getFFvalue($this->cObj->data['tx_kkcsv2table_flex'], 'utf8_encoded', 'sDEF'));

  		if ($this->debug) {
         print '<p style="color:#ad2121;">F L E X F O R M - D A T A :<br>';
         print '$csv_file = '.$csv_file.'<br />';
         print '$col_delimiter = '.$this->col_delimiter . " = x'".chunk_split(bin2hex($this->col_delimiter), 2, '')."'<br />";
         print '$text_delimiter = '.$this->text_delimiter.'<br />';
         print '$csv_code = '.$this->csv_code.'<br />';
         print '$utf8_encoded = '.$this->utf8_encoded.'<br />';
         print '$table_summary = '.$table_summary.'<br />';
         print '$table_class = '.$table_class.'<br />';
         print '$header_data = '.$header_data.'<br />';
         print '$own_header = '.$own_header.'<br />';
         print '</p>';
      }


      // START of  C O N T E N T:
      $content = '<!-- begin of block extension "'.$this->extKey.'" -->';

      // Open and read data from flat-file (csv or txt)
      if (($fp = fopen ($csv_file , 'r' )) == false) {
         return '<div style="color:red; font-weight:bold;" class="'.$extKey.'">'.$this->pi_getLL('error404').'<br />'.$csv_file.'</div>';
      }

      // analyse the maximum number of columns
      $this->max_cols = $this->max_number_of_cols($csv_file);
      if ($this->debug) print '<p style="color:#2121ad;">Max number of columns = "' . $this->max_cols . '"</p>';


      // put in dedicated parameters for the table-tag
      $table_param = '';
      if ($table_class != '')   $table_param .= ' class="'.$table_class.'"';
      if ($table_summary != '') $table_param .= ' summary="'.$table_summary.'"';

      $content .= '<div class="'.$this->extKey.'"><table'.$table_param.'>';

     	$this->oe = 0;    // indicates the special class of "$oddEvenArray[$this->oe]"
      // own header-string, same line construction will be added on top
      $ownHeaderString = (!empty($own_header)) || ($own_header <> '');
      if ($ownHeaderString) {
         // If a tabulator ist the col-delimiter!?
         $own_header = str_replace('[t]', $this->col_delimiter, $own_header);
         $own_header = str_replace('[T]', $this->col_delimiter, $own_header);
	      $line = str_getcsv($own_header, $this->col_delimiter, $this->text_delimiter);
         $content .= $this->createRow($line, 'th') ;
      }

      // if header-data is set, the first row of csv will be the col-headers (possibly the 2nd id also own_header is filled
      if ($header_data) {
         $line = fgetcsv($fp,4096,$this->col_delimiter,$this->text_delimiter);
         if (!$ownHeaderString) {
            $content .= $this->createRow($line, 'th') ;
         }
      }


      // loop over all table-rows
      $this->oe = 2;          // switcher for odd or even - class
      while ($line = fgetcsv($fp,4096,$this->col_delimiter,$this->text_delimiter)) {
         // "if" because of possible last empty row
         if ($line != '') {
            $this->oe = 3 - $this->oe;
      	   $content .= $this->createRow($line, 'td');
      	}
      }

      $content .= '</table></div>    <!-- end of block extension "'.$this->extKey.'" -->';

      fclose($fp);

//		return $content;
      return $this->pi_wrapInBaseClass($content);
	}


	/**
	 * Create one table-row
	 *
	 * @param	array of string	  $r: row-data-array
	 * @param	string	           $celltype: e.g. "td" or "th"
	 * @return  string              complete <tr>...</tr> - row
	 */
	function createRow($r, $celltype){
      $c = '<tr class="'.$this->oddEvenArray[$this->oe].'">';


      $cols = count($r);
      for($x=0; $x < $cols; $x++) {
         if ($this->debug > 1) print " /// Z$x = $r[$x]";

         // check HTML-Tags in input string:
         $r[$x] = $this->allowedTags_encode($r[$x]);


         $tag = '<'.$celltype.' class="'.$celltype.'-'.$x.'">';
         // last cell (col)!?
         if ($x == $cols - 1) {
            $colspan = $this->max_cols - $cols + 1;
            if ($colspan > 1) {
               $tag = '<'.$celltype.' class="'.$celltype.'-last" colspan="'.$colspan.'">';
            }else{
               $tag = '<'.$celltype.' class="'.$celltype.'-last">';
            }
         }

         if ( !empty($this->utf8_encoded) )  $r[$x] = utf8_encode($r[$x]);

         $r[$x] = $this->allowedTags_decode($r[$x]);


         $c .= $tag . $r[$x] . "</$celltype>";

      }
      
      $c .= '</tr>';
   return $c;
   }

	/**
	 * Checks the allowed HTML-Tags ($conf)) in the string
	 *
	 * @param	string	$p: row-data-string
	 * @return  string   cleaned from TAGS except those that are allowed ("allowTags" plus <br />)
	 */
	function allowedTags_encode($p){

      $w = $p;
      $w = str_replace('<br>','~1~br /~2~', $w);
      $w = str_replace('<br />','~1~br /~2~', $w);
      $w = str_replace('<br/>','~1~br /~2~', $w);
      // die Kombinationen mssen zuerst, da die Klammern ja sonst schoon substituiert wren:
      $w = str_replace(chr(60).chr(47),'~TAGEND~', $w);
      $w = str_replace(' '.chr(47).chr(62),'~sTAGEND~', $w);
      $w = str_replace(chr(60),'~[~', $w);
      $w = str_replace(chr(62),'~]~', $w);
      $at = explode(',', $this->conf['allowTags']);
      if ($this->debug > 1) print '<p style="color:#f4e;">raw string = '. $w .'<br />"'.chunk_split(bin2hex($w), 2, ' ') . '"</p>';
   	for($i = 0; $i < count($at); $i++) {
         $w = str_replace('~[~'.$at[$i],'~1~'.$at[$i], $w);
         $w = str_replace('~TAGEND~'.$at[$i],'~3~'.$at[$i], $w);
         $w = str_replace($at[$i].'~]~',$at[$i].'~2~', $w);
         $w = $this->setEndTag($w, $at[$i]);
      }

      $w = str_replace('~[~','<', $w);
      $w = str_replace('~]~','>', $w);
      $w = str_replace('~TAGEND~', '</', $w);

      if ( empty($this->csv_code)) {
         $w = htmlentities($w);
      } else {
         $w = htmlentities($w, ENT_COMPAT, $this->csv_code );
      }

      return $w;
   }

	/**
	 * Checks the allowed HTML-Tags ($conf)) in the string
	 *
	 * @param	string	$p: row-data-string
	 * @return  string   cleaned from TAGS except those that are allowed ("allowTags" plus <br />)
	 */
	function setEndTag($p, $needle) {
      $w = $p;
      $i = strpos($w, '~1~'.$needle);
      while ( $i > -1 ) {
         $j = strpos($w, '~]~', $i+2);
         if ($j > $i) $w[$j+1] = '2';
         $i = strpos($w, '~1~'.$needle, $i+1);
      }
      return $w;
   }


	/**
	 * Checks the allowed HTML-Tags ($conf)) in the string
	 *
	 * @param	string	$p: row-data-string
	 * @return  string   cleaned from TAGS except those that are allowed ("allowTags" plus <br />)
	 */
	function allowedTags_decode($p){
      $w = $p;

      $w = str_replace('~1~','<',$w);
      $w = str_replace('~2~','>',$w);
      $w = str_replace('~3~','</',$w);
      $w = str_replace('~sTAGEND~', ' />', $w);
      $w = str_replace("\n", '<br />', $w);

      return $w;
   }



	/**
	 * find the maximum  number of columns in the hole file:
	 *
	 * @param	string	$f: input file name
	 * @return  integer  maximum number of columns over all rows
	 */
	function max_number_of_cols($f) {
      ($file_handle = fopen ($f , 'r' )) or die($this->pi_getLL('error404').' "'.$f.'"!?');

      $max = 0;
      while ($r = fgetcsv($file_handle,4096,$this->col_delimiter)) {
         $max = max($max, count($r));
      }

      fclose($file_handle);

      return $max + 1;
   }

}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/kk_csv2table/pi1/class.tx_kkcsv2table_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/kk_csv2table/pi1/class.tx_kkcsv2table_pi1.php']);
}

?>
